# 问题

一渔夫以钓鱼谋生。当地鱼竿的售价为 400 元/根（不论新旧），租价为 100 元/根/天；鱼的售价为 500 元/筐；渔夫的雇佣费为 200 元/个/天，每个渔夫每天不能同时使用 2 根及以上的鱼竿且只能钓到 1 筐鱼。租鱼竿和雇佣渔夫都只能预付费。
该渔夫最初只有 1 筐鱼，忽略鱼、鱼竿、渔夫的交易耗时，求 3 天后该渔夫售鱼的最大所得。

# 问题分析

1、渔夫和鱼竿都是存量资源，只有鱼是增量资源，因此收益取决于 3 天能钓到多少鱼。

2、渔夫的雇佣费为 200 元/个/天，而当事人本身是渔夫，因此他自身的增值是 200 元/天。

3、钓 1 筐鱼的成本 = 鱼竿成本 + 渔夫人工费，渔夫人工费为固定的 200 元，而鱼竿成本有购买和租两种情况。若鱼竿是租的，则鱼竿成本为 100 元/天；若鱼竿是买的，由于用完可以再卖，因此鱼竿成本为 0 元/天。因此钓 1 筐鱼的成本和收益为：

* 若租鱼竿：成本为 200 + 100 = 300 元，收益为 500 - 300 = 200 元，所需启动资金为 300 元
* 若买鱼竿：成本为 200 + 400 - 400 = 200 元，收益为 500 - 200 = 300元，所需启动资金为 600 元

4、什么情况下应该买鱼竿，什么情况下应该租鱼竿？

由于所需启动资金分为 300 元和 600 元 2 种情况，因此：

* 当 0 < 资金 < 300 时：什么也做不成
* 当 300 == 资金时：只能租，收益为 200 元
* 当 300 < 资金 < 600 时：花 300 元租鱼竿和渔夫, 剩余的钱什么也做不成，收益为 200 元
* 当 600 == 资金时：买鱼竿+租渔夫，用完再把鱼竿卖掉，收益为 300 元
* 当 600 < 资金时：以 600 元为单位，把资金分割成一份一份，每份单独计算

注意：须将渔夫渔夫每日增值的 200 元添加到总资金里面。

5、由于忽略鱼、鱼竿、渔夫的交易耗时，即：鱼、鱼竿、渔夫可以随时找到交易对象，因此 3 天的收益可以拆分开，一天一天算，先计算第 1 天的最高收益，再以第 1 天的结算为基础，计算第 2 天的最高收益……以此递归，可以计算第 n 天的最高收益。

# 代码

```python
鱼竿售价 = 400
每筐鱼售价 = 500
渔夫每日增值 = 200

def 计算当日结余(可用资金):
    可用资金 += 渔夫每日增值
    预收入 = 0
    # 当 600 == 可用资金时：买鱼竿+租渔夫
    六百元份数 = 可用资金 // 600
    可用资金 -= (六百元份数 * 600)  # 劳动成本为预付费
    预收入 += (每筐鱼售价 * 六百元份数)  # 劳动结束时才有收入, 因此为'预收入'
    # 当 300 <= 可用资金 < 600 时：只能租
    if 可用资金 // 300:
        可用资金 -= 300
        预收入 += 每筐鱼售价
    # 劳动结束, 把购买的鱼竿卖出去, 并计算当日结余
    结余 = 可用资金 + 预收入 + 鱼竿售价 * 六百元份数
    return 结余

累计收益 = 0
可用资金 = 每筐鱼售价  # 该渔夫最初只有 1 筐鱼
for i in range(1, 11):
    # 第 i 天
    结余 = 计算当日结余(可用资金)
    收益 = 结余 - 可用资金
    累计收益 += 收益
    print(f"第 {i} 天, 收益为 {收益} 元, 累计收益为 {累计收益} 元")
    可用资金 = 结余  # 第 i 天的结余就是第 i+1 天的可用资金
```

打印结果：

```
第 1 天, 收益为 500 元, 累计收益为 500 元
第 2 天, 收益为 800 元, 累计收益为 1300 元  
第 3 天, 收益为 1100 元, 累计收益为 2400 元 
第 4 天, 收益为 1700 元, 累计收益为 4100 元 
第 5 天, 收益为 2600 元, 累计收益为 6700 元 
第 6 天, 收益为 3800 元, 累计收益为 10500 元
第 7 天, 收益为 5800 元, 累计收益为 16300 元
第 8 天, 收益为 8600 元, 累计收益为 24900 元
第 9 天, 收益为 13000 元, 累计收益为 37900 元
第 10 天, 收益为 19400 元, 累计收益为 57300 元
```
